/**
 * Constants for GNet. Must match the C code!
 * @file gproxy/Constants.java
 **/
package gproxy;

/**
 * Constants for GNet. Must match the C code!
 * @author Christian Grothoff
 **/
public class Constants {

    /**
     * maximum frequency for requests 
     **/
    public final static int MIN_DELAY = 30000; // 30s

    /**
     * Default TTL: 30s
     **/
    public final static int TTL_DEFAULT = 30;

    /**
     * How many outstanding requests max (per download simultaniously)
     **/
    public final static int MAX_REQUESTS = 250;    
 
    /**
     * Give up after how many requests without reply?
     **/
    public final static int REQ_LIMIT = 10;
    
    /** ******************** these must match the C code! *************** **/

    /**
     * Size of content blocks in GNet.
     **/ 
    public final static int CONTENT_BLOCK_SIZE=1024;

    /**
     * Size of the hashcodes.
     **/
    public final static int HASH_SIZE = 20;/* 160 bits */

    /**
     * Offset of prirotiy in Message to GNet
     **/
    public final static int PRIORITY_OFFSET = 0;

    /**
     * Offset of time-to-live in Message to GNet
     **/
    public final static int TTL_OFFSET = 4;

    /**
     * Offset of prirotiy in Message to GNet
     **/
    public final static int HASH_OFFSET = 8;

    /**
     * What is the maximum number of entries in each inode?
     **/
    public final static int NODES_PER_INODE  = 51;

    /**
     * What is the size of the leaf nodes?
     **/
    public final static int LEAF_SIZE = 1024;

    /**
     * At which offset in the root-node is the
     * hash-value stored? 
     **/
    public final static int ROOT_HASH_OFFSET = 8;

    /**
     * At which offset is the hash value in the
     * root-node stored?
     **/
    public final static int ROOT_CRC32_OFFSET = 4;

    /**
     * At which offset in the root node is the length
     * of the description stored?
     **/
    public final static int ROOT_DESCLEN_OFFSET = 28;

    /**
     * At which offset in the root-node is the description stored?
     **/
    public final static int ROOT_DESC_OFFSET = 30;

    /**
     * At which offset in the root-node is the
     * file-length stored?
     **/
    public final static int FILE_LENGTH_OFFSET = 0;
      
} // end of Constants
