/**
 * @file gproxy/DownloadModel.java
 **/
package gproxy;

import java.util.Vector;

/**
 * This Model represents the active downloads.
 * @author Christian Grothoff
 **/
public class DownloadModel 
    extends javax.swing.AbstractListModel {
    
    /**
     * Frame displaying active downloads.
     **/
    private DownloadFrame dlf_;

    /**
     * active pending downloads systemwide
     **/
    private final Vector downloads_;

    /**
     * Request manager for new downloads.
     **/
    private final RequestManager rm_;

    /**
     * Create a new QueryModel.
     **/
    public DownloadModel(RequestManager rm) {
	super();
	this.rm_ = rm;
	this.downloads_ = new Vector();
    }

    /**
     * Add the given node to the pending downloads. 
     * Also start the download.
     * @param node the node to download
     **/
    public synchronized void download(RootNode node) {
	// better: open file-save-as dialog here
	// (in separate Thread), the Result 
	// should then be used as the filename
	String fileName =  null;
	// swing:
	javax.swing.JFileChooser fc 
	    = new javax.swing.JFileChooser(".");
	int returnVal = fc.showSaveDialog(this.dlf_);
	if (returnVal != javax.swing.JFileChooser.APPROVE_OPTION) 
	    return;
	fileName = fc.getSelectedFile().getName();
	
	RNode rn = node.getDownloadNode(fileName);
	downloads_.add(rn);
	rn.download(rm_, this);
	// update view
	if ((dlf_ == null))
	    dlf_ = new DownloadFrame(this);	    	
	fireIntervalAdded(this,
			  downloads_.indexOf(rn),
			  downloads_.indexOf(rn));
	if (GProxy.DEBUG)
	    System.out.println("Starting download: " + node.toString());    
    }

    /**
     * Update the display.
     **/
    public synchronized void update(RNode who) {
	int index = downloads_.indexOf(who);
	fireContentsChanged(this,
			    index, index);
    }

    /**
     * Abort the downloads given by their indices.
     **/
    public synchronized void abort(int[] what) {
	for (int i=0;i<what.length;i++) {
	    // locate root-node
	    RNode rn = (RNode) downloads_.elementAt(what[i]);
	    // remove from query list,
	    downloads_.remove(rn);
	    // update view
	    fireIntervalRemoved(this,what[i],what[i]);
	    // add to download list
	    rn.abort();
	}
    }

    /**
     * Download window closed.
     **/
    public synchronized void windowClosed() {
	dlf_ = null;
    }

    /**
     * Return the n-th RNode to display for this QM.
     **/
    public Object getElementAt(int i) {
	return downloads_.elementAt(i);
    }

    /**
     * Return the current number of matching search results.
     **/
    public int getSize() {
	return downloads_.size();
    }

    /**
     * Notification that some RootNode has completed the download.
     **/
    public synchronized void notifyComplete(RNode who) {
	int index = downloads_.indexOf(who);
	downloads_.remove(who);
	fireIntervalRemoved(this,index,index);
    }

} // end of DownloadModel
