/**
 * GNet Debugging module. Configure here for which components you
 * want extra debugging output.
 * @author Christian Grothoff
 * @file include/debugging.h
 **/

#ifndef DEBUGGING_H
#define DEBUGGING_H

#include "storage.h"

/**
 * Where to print errors to?
 **/
#define PRINTTO stderr

#define EXTRA_CHECKS 1 

/* #define LOAD_DEBUG 1*/

/**
 * Print all important UDP actions.
 **/
/* #define PRINT_UDP 2 */

/**
 * Print absolutely all UDP actions.
 **/
/* #define PRINT_UDP_ALL 2 */

/**
 * Print all important TCP actions.
 **/
/* #define PRINT_TCP 2 */

/**
 * Print absolutely all TCP actions.
 **/
/* #define PRINT_TCP_ALL 2 */

/**
 * Print policy decisions.
 **/
#define PRINT_POLICY 1  

/**
 * Print the actual contents of each packet
 **/
/* #define PRINT_PACKETS 0 */

/**
 * Print all thread actions (threads starting and stopping
 * to work).
 **/
/* #define PRINT_THREADS 2 */

/**
 * Print all configuration actions.
 **/
/* #define PRINT_CONFIGURATION 1 */

/**
 * Print all warnings about detected errors in incoming
 * data, e.g. malformed packets (CRC).
 * THIS SHOULD BE KEPT, EVEN FOR PRODUCTION RELEASES.
 **/
#define PRINT_WARNINGS 1

/**
 * Print extra information about the CRC checks.
 **/
/* #define PRINT_CRC 0 */

/**
 * Print extra information about the CRC checks.
 **/
/* #define PRINT_CRC_ALL 0  */

/**
 * Print information about HELO activities.
 **/
/* #define PRINT_HELO 1  */

/**
 * Print extra messages in the identity module
 **/
/* #define PRINT_IDENTITY 1  */

/**
 * Print information in the lookup module
 **/
/* #define PRINT_LOOKUP 1 */

/**
 * Display the lookup table in lookupCron
 **/
/* #define PRINT_LOOKUP_TABLE 1 */

/**
 * Print add entry information in the lookup module
 **/
/* #define PRINT_LOOKUP_INIT 1 */

/**
 * Should ASSERTS be checked?
 * THIS SHOULD BE KEPT, EVEN FOR PRODUCTION RELEASES.
 **/
#define CHECK_ASSERTS 1 

/**
 * Should HELO trigger a (test) Query?
 **/
/* #define DEBUG_HELO_TRIGGERS_QUERY 1 */

/**
 * Dump packet summaries.
 **/
/* #define DUMP_PACKETS 1 */

/**
 * Debug sessionkey events.
 **/
/* #define PRINT_SKEY 3 */

/**
 * Debug sessionkey events.
 **/
/* #define PRINT_SKEY_ALL 1 */

/**
 * Debug sessionkey events.
 **/
/* #define PRINT_QUERY 2  */

/**
 * Debug sessionkey events.
 **/
/* #define PRINT_QUERY_ALL 1  */

/**
 * Debug sessionkey events.
 **/
/* #define PRINT_CONTENT 3 */

/**
 * Debug sessionkey events.
 **/
/* #define PRINT_CONTENT_ALL 1 */

/**
 * Debug sessionkey events.
 **/
/* #define PRINT_ROUTING 2  */

/**
 * Debug connection module.
 **/
/* #define PRINT_CONNECTION 2  */

/**
 * Debug connection module.
 **/
/* #define PRINT_CONNECTION_ALL 1 */

/**
 * Run without control-thread. NEVER do except for
 * debugging with fewer thread issues!
 **/
/* #define DISABLE_CONTROL_THREAD 0 */

/**
 * Run without CRC checking
 **/
/* #define DISABLE_CRC_CHECK 1 */


/* ********************** Types ***************** */

/* IP:Port as a string */
typedef struct {
  char ipport[4*4+5+1]; 
} IPPortString;

/* IP as a string */
typedef struct {
  char ip[4*4+1]; 
} IPString;

/* Sessionkey as a string */
typedef struct {
  char skey[5*SESSIONKEY_LEN+1]; 
} SKEYString;



/**
 * errexit - print an error message and exit
 * @param format the string describing the error message
 **/
void errexit(const char *format, ...);

/**
 * errexit - print an error message and exit
 * @param format the string describing the error message
 **/
void print(const char *format, ...);

/**
 * Display the contents of an incoming UDP message.
 **/
void showMessageEncrypted(MessagePack * msg);

/**
 * Display the contents of an incoming UDP message.
 **/
void showMessageDecrypted(MessagePack * msg);

/**
 * Print the connection address (IP and Port) encoded in the given
 * HostAddress to stderr.
 * @param conInfo the HostAddress to print.
 **/
void printAddress(HostAddress * conInfo,
		  IPPortString * result);
 
/**
 * Print the connection address (IP) encoded in the given
 * integer.
 * @param theIP the encoded IP
 **/
void printIP(int theIp,
	     IPString * result);

void printSessionKey(SESSIONKEY * key,
		     SKEYString * result);

/**
 * Assert something. Display message if it fails.
 **/
void assert(int condition,
	    char * message);

/* ************************ active DEBUG ************** */

/**
 * The following is a testing routine that will execute sending
 * of a query to a bunch of other hosts
 */
void DEBUG_broadcastQuery();

/**
 * Print the given packet in a human-readable form
 * (especially giving the parts of the packet).
 **/
void dumpPacket(unsigned char * message,
		BLOCK_LENGTH len);





#endif
/* end of debugging.h */
