/**
 * insertFile - breaks file meant for insertion into blocks, prepares
 *              for insertion into network
 * @author Krista Bennett
 * @file gnet/include/insertfile.h
 **/

#ifndef INSERT_FILE_H
#define INSERT_FILE_H

#include "gnettypes.h"
#include "storage.h"
#include "checksum.h"
#include "symcipher.h"

/**
 * Inserts a file under the given name into the local gnet node.
 * @param filename the name of the (incoming/source) file
 * @param description the description of the file
 * @param keywords the keywords that shall be used to retrieve the file
 * @param num_keys the number of keywords we are to associate with the file
 * @param encode_content: 1, if we are going to encrypt, encode, and write      
 *                        the content portions themselves 
 *                        0, if we are simply going to create the indirection 
 *                        nodes for the file (for on-demand encoding later)
 * @return 0: ok, -1: file not found
 **/
int insertFile(FileName filename,
               unsigned char * description,
               char ** keywords,
               int num_keys,
               unsigned int encode_content); 


#endif
