/**
 * sessionkey management
 * @author Christian Grothoff
 * @file include/sessionkey.h
 **/

#ifndef SESSIONKEY_H
#define SESSIONKEY_H

#include "gnettypes.h"
#include "identity.h"
#include "symcipher.h"

/**
 * Force creation of a new Session key for the given host. 
 * @param hostId the identity of the other host 
 * @param sk the SESSIONKEY to use
 * @param created the timestamp to use
 * @param ret the address where to write the signed session key, first BLOCK_LENGTH_SIZE byte give length
 * @return 1 on success, 0 on failure
 **/
int makeSessionKeySigned(HostIdentity * hostId,
			 SESSIONKEY * sk,
			 time_t created,
			 SKEY_Message * ret);

/**
 * @return 0 if invalid, 1 if valid
 **/
int verifySKS(HostIdentity * hostId,
	      SKEY_Message * sks);

/**
 * Accept a session-key that has been sent by another host.
 * The other host must be known (public key)
 * @param hostId the identity of the sender host
 * @param sessionkey the session key that was "negotiated"
 * @param sig the signature to verify the sessionkey is ok
 **/
void acceptSessionKey(HostIdentity * hostId,
		      SKEY_Message * sessionkeySigned);

/**
 * The following routine constructs a receipt for the 
 * session key.
 * @param sessionkey the key to use
 * @param ret address where to write the receit to
 **/
void makeAckMessage(SessionKeyEncoded * sessionKey,
		    ACK_Message * ret);

#endif
/* end of sessionkey.h */
