/**
 * Generic cron-jobs. Currently contains only HELO
 * exchanges.
 * @author Christian Grothoff
 * @file gnet/primitives/cronJobs.c
 **/

#include "cronJobs.h"

/**
 * The DirContents array for scanning the hosts/ directory.
 **/
extern DirContents * CONNECTION_knownHosts;
extern int CONNECTION_currentMaxKnownHosts;
extern int CONNECTION_currentKnownHosts;
extern Semaphore * CONNECTION_knownHostsLock;
extern DirName * CONNECTION_networkIdDirectory;

/* ************* internal Methods **************** */

/**
 * Tell everybody we are there...
 **/
void broadcastHELO();
 
/**
 * Forward HELOs from all known hosts to all known hosts.
 **/
void forwardHELO();

/* ******************** CODE ********************* */

/**
 * initialize a few cron jobs. Must be called after
 * initcron (!).
 **/
void initCronJobs() {
  addCronJob(&broadcastHELO,
	     30*60,30*60); /* every 30 minutes */
  addCronJob(&broadcastHELO,
	     60*60,60*60); /* every hour */
}


/**
 * Tell everybody we are there...
 **/
void broadcastHELO(){
  int i, crc;
  HostIdentity myId, hostId;
  HELO_Message m;

  createHELO(&m);
  crc = crc32N(&m,sizeof(HELO_Message));
  getHostIdentity(getPublicHostkey(),&myId);  

  semaphore_down(CONNECTION_knownHostsLock);   
  for (i=0;i<CONNECTION_currentKnownHosts;i++) {
    hex2hostIdentity(&CONNECTION_knownHosts->files[i], 
		     &hostId);
    if (hostIdentityEquals(&myId,&hostId)){
      i++;
      continue;
    }
    sendToHost(&hostId,
	       sizeof(HELO_Message),
	       &m.header,
	       0, crc);   
    i++;    
  }
  semaphore_up(CONNECTION_knownHostsLock);   
}

/**
 * helper method for forwardHELO to all hosts without encryption!
 **/
void fwdHelo(HELO_Message * m) {
  time_t current;
  HostIdentity myId, hostId;
  int i;

  time(&current);
  if (m->body.expires < current)
    return; /* message did expire, do not send! */
  getHostIdentity(getPublicHostkey(), &myId);  
  for (i=0;i<CONNECTION_currentKnownHosts;i++) {
    hex2hostIdentity(&CONNECTION_knownHosts->files[i], 
		     &hostId);
    if (hostIdentityEquals(&myId,&hostId) || 
	hostIdentityEquals(&hostId,&m->body.senderIdentity)){
      i++;
      continue;
    }
    unicast(&m->header,&hostId,0);
    i++;
  }
}

/**
 * Forward HELOs from all known hosts to all known hosts.
 **/
void forwardHELO(){
  HELO_Message fi;
  FileName fn;
  int i;

  fi.header.size = sizeof(HELO_Message) - BLOCK_LENGTH_SIZE;
  fi.header.requestType = GNET_PROTO_HELO; 
  fn = malloc(strlen((char*)CONNECTION_networkIdDirectory)+sizeof(HexName)+1);

  semaphore_down(CONNECTION_knownHostsLock);   
  for (i=0;i<CONNECTION_currentKnownHosts;i++) {
    buildFileName(CONNECTION_networkIdDirectory,
		  &CONNECTION_knownHosts->files[i],
		  fn);
    readFile(fn, sizeof(HELO_Body), &fi.body);
    fwdHelo(&fi);
  }
  semaphore_up(CONNECTION_knownHostsLock);   
  free(fn);
}

/* end of cronJobs */
