/**
 * This is the testcase for the connection module.
 * @author Christian Grothoff
 * @file gnet/database/test/connectiontest.c
 **/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <pthread.h>

#include "connection.h"
#include "keyservice.h"
#include "identity.h"

#define TESTSTRING "Hello World\0"

int ok;
int failureCount = 0;

void testConnectionLoopback() {
  CONTENT_Message message;
  HELO_Message msg;

  createHELO(&msg);
  bindAddress(&msg);
  sleep(1);
  memcpy(&message.reply, TESTSTRING, strlen(TESTSTRING)+1);
  broadcast(&message.header, 0, 4);
  cronConnection();
  cronConnection();
  cronConnection();
  sleep(1);
  kill(ok,9); 
}

/*------------------------------------------------------------------------
 * passivesock - allocate & bind a server socket using GNET-UDP
 *------------------------------------------------------------------------
 */
int passivesock() {
  struct sockaddr_in sin;	/* an Internet endpoint address		*/
  int	s, type;	/* socket descriptor and socket type	*/
  memset(&sin, 0, sizeof(sin));
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = htons(getGnetPort());

  /* UDP */
  type = SOCK_DGRAM;
  /* Allocate a socket */
  s = socket(PF_INET, type, 17 /* UDP */);
  if (s < 0) 
    errexit("can't create socket: %s\n", strerror(errno)); 
  
  /* Bind the socket */
  if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) 
    errexit("can't bind to port: %s\n",
	    strerror(errno));
  return s;
}

void startListener() {
  struct sockaddr_in incomming;
  int socket = passivesock();
  int addrlen = sizeof(incomming);  
  int size;
  unsigned char message[1500];
  char result[1500];
  MessagePack * pack;
  BLOWFISHEncryptedData * data;
 
  if (0 == (ok=fork())) {
    testConnectionLoopback();
    exit(0);
  }
  bzero(&incomming,sizeof(struct sockaddr_in));
  if (socket < 0) {
    errexit("unknown error reading from socket (%d)\n",
	    socket);
  }
  size = recvfrom(socket, message, MESSAGE_SIZE, 0,
		  (struct sockaddr * )&incomming, &addrlen);  
  close(socket);

  pack = (MessagePack *) message;
  pack->size = pack->size & 0x7FFF; /* mask out highest bit */
  data = (BLOWFISHEncryptedData*) &pack->parts[0];
  /* decrypt msg */
  if (pack->size+BLOCK_LENGTH_SIZE != size) 
    errexit("package claims to be %d bytes but I got %d bytes!\n",
	    pack->size+BLOCK_LENGTH_SIZE, size);  
  if (-1 == decryptFromHost(data,
			    pack->size - sizeof(MessagePack),
			    &pack->senderIdentity,
			    result))
    errexit("Can not decrypt packet: no session key!\n");

  /* we would fail here, I currently assume that we're looking
     at the SKEY exchange and not the actual message here... */
  print("testConnectionLoopback ok\n");
  exit(0);
}

int main(int argc, char * argv[]) {
  readConfig(DEFAULT_CONFIG_FILE);
  initAddress();
  initKeyService();
  initConnection();
  startListener();

  if (failureCount == 0)
    return 0;
  else {
    printf("\n\n%d TESTS FAILED!\n\n",failureCount);
    return -1;
  }
} // end of main
