/**
 * Hostkey - management of the node's main identity.
 * This is the testcase!
 * @author Christian Grothoff
 * @file gnet/database/test/hostkeytest.c
 **/

#include <stdio.h>
#include <string.h>
#include "hostkey.h"

#define TESTSTRING "Hello World\0"
#define MAX 20

int testEncryptDecrypt() {
  HOSTKEY hostkey;
  PublicKey pkey;
  RSAEncryptedData target;
  char result[MAX];

  hostkey = makeHostkey();
  getPublicKey(hostkey, &pkey);
  if (0 == encrypt(TESTSTRING,strlen(TESTSTRING)+1,&pkey,
		   &target))
    return 1;
  if (-1 == decrypt(hostkey, &target, result, MAX))
    return 1;
  freeHostkey(hostkey);
  if (strcmp(TESTSTRING,result) == 0) {
    printf("testEncryptDecrypt ok\n");
    return 0;
  } else {
    printf("%s != %s - testEncryptDecrypt failed!\n",
	   TESTSTRING,result);
    return 1;
  }
}

int testSignVerify() {
  HOSTKEY hostkey;
  Signature sig;
  PublicKey pkey;

  hostkey = makeHostkey();
  getPublicKey(hostkey, &pkey);
  if (0 == sign(hostkey, strlen(TESTSTRING), TESTSTRING, &sig))
    return 1;
  if (1 == verifySig(TESTSTRING, strlen(TESTSTRING), &sig, &pkey)) {
    printf("testSignVerify ok\n");
    return 0;
  } else {
    printf("testSignVerify failed!\n");
    return 1;
  }
}

int testHostkeyEncoding() {
  HOSTKEY hostkey;
  HostKeyEncoded * encoding;
  PublicKey pkey;
  RSAEncryptedData target;
  char result[MAX];

  hostkey = makeHostkey();
  getPublicKey(hostkey, &pkey);
  if (0 == encrypt(TESTSTRING,strlen(TESTSTRING)+1,&pkey,
		   &target))
    return 1;
  encoding = encodeHostkey(hostkey);
  freeHostkey(hostkey);
  hostkey = decodeHostkey(encoding);
  free(encoding);
  if (-1 == decrypt(hostkey, &target, result, MAX))
    return 1;
  freeHostkey(hostkey);
  if (strcmp(TESTSTRING,result) == 0) {
    printf("testHostKeyEncoding ok\n");
    return 0;
  } else {
    printf("%s != %s - testHostkeyEncoding failed!\n",
	   result, TESTSTRING);
    return 1;
  }
}

int main(int argc, char * argv[]) {
  int failureCount = 0;

  failureCount += testEncryptDecrypt(); 
  failureCount += testSignVerify();
  failureCount += testHostkeyEncoding(); 

  if (failureCount == 0)
    return 0;
  else {
    printf("\n\n%d TESTS FAILED!\n\n",failureCount);
    return -1;
  }
} // end of main
